package ps6.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import ps6.model.Board;
import ps6.model.ShuffleListener;

/**
 * 
 * 
 * @author mbolin
 */
public class BoardPanel extends JPanel implements ShuffleListener {

   public static final long serialVersionUID = 61702;

	private Board board;

	private JLabel[][] labels;

	private static final int LABEL_WIDTH = 100;

	private static final int GUTTER_WIDTH = 2;

	private static final Font CUBE_FONT = new Font(null, Font.BOLD, 24);
	
	public BoardPanel(Board board) {
		this.board = board;
		setLayout(new GridLayout(board.getBoardSize(), board.getBoardSize(),
				GUTTER_WIDTH, GUTTER_WIDTH));
		labels = new JLabel[board.getBoardSize()][board.getBoardSize()];
		for (int row = 0; row < board.getBoardSize(); row++) {
			for (int col = 0; col < board.getBoardSize(); col++) {
				JLabel label = new JLabel(board.getLettersAt(row, col));
				label.setPreferredSize(new Dimension(LABEL_WIDTH, LABEL_WIDTH));
				label.setFont(CUBE_FONT);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				label.setBackground(Color.WHITE);
				labels[row][col] = label;
				add(label);
			}
		}
		setPreferredSize(new Dimension(board.getBoardSize() * LABEL_WIDTH,
				board.getBoardSize() * LABEL_WIDTH));
		setBackground(Color.ORANGE);
	}
	
	public void boardShuffled(Board b) {
		if (b != board) {
			return; // should only be listening for shuffle events for my Board 
		}
		updateLetters();
	}
	
	private void updateLetters() {
		for (int row = 0; row < board.getBoardSize(); row++) {
			for (int col = 0; col < board.getBoardSize(); col++) {
				JLabel label = labels[row][col];
				label.setText(board.getLettersAt(row, col));
			}
		}
	}

}
