package ps6.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import ps6.model.Board;

/**
 * @author mbolin
 */
public class InputPanel extends JPanel {

    public static final long serialVersionUID = 61703;

    private JButton shuffleButton;

    private JTextField entry;

    private JButton submitButton;

    private JList wordList;
    
    private DefaultListModel model;
    
    private final Board board;

    public InputPanel(Board b) {
        this.board = b;

        setLayout(new BorderLayout());

        //---------------------------
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BorderLayout());
        shuffleButton = new JButton("Shuffle");
        shuffleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                board.shuffle();
                model.clear();
            }
        });
        topPanel.add(shuffleButton, BorderLayout.NORTH);

        submitButton = new JButton("Submit Word");
        submitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                checkWord();
            }
        });
        topPanel.add(submitButton, BorderLayout.SOUTH);

        entry = new JTextField();
        entry.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e) {
               checkWord();
           }
        });
        topPanel.add(entry, BorderLayout.CENTER);
        add(topPanel, BorderLayout.NORTH);
        //----------------------------

        //----------------------------
        model = new DefaultListModel();
        wordList = new JList(model);        
        JScrollPane scrollPane = new JScrollPane(wordList);
        scrollPane.setPreferredSize(new Dimension(50, 30));
        add(scrollPane, BorderLayout.CENTER);
        //----------------------------
        
        setBackground(Color.WHITE);
    }
    
    private void checkWord() {
        String word = entry.getText();
        if (word.length() < 3) {
            JOptionPane.showMessageDialog(getParent(),
                    "Words must be at least 3 letters long.");
            return;
        }
        if (board.containsWord(word)) {
            if (!(model.contains(word))) {
                model.addElement(word);
            }
            entry.setText("");
        } else {
            JOptionPane.showMessageDialog(getParent(),
                    "Board does not contain " + word.toUpperCase());
        }
    }

}
