package ps6.model;

/**
 * <code>Board</code> represents a square Boggle board.
 * 
 * Like a real Boggle board, it can be shuffled
 * and the letters on the face of each cube in the grid can be viewed.
 *
 * @specfield String [][] cubeFaceStrings // the strings that appear in the board positions
 * @specfield (set ShuffleListener) shuffleListeners // those who listen for shuffle events
 *
 * abstract invariant: cubeFaceStrings is a square matrix
 * 
 * @author mbolin
 */
public interface Board {

    /**
     * @returns the length of the side of the board.
     */
	public int getBoardSize();
	
	/**
	 * @modifies cubeFaceStrings
	 * @effects shuffles this <code>Board</code> and notifies all
	 *          ShuffleListeners registered with this <code>Board</code> that
	 *          the shuffle has occurred
	 */
    public void shuffle();

    /**
     * @returns the letters on the cube at the specified position
     * @throws IllegalArgumentException if
     *   (!(0 <= row < getBoardSize() && 0 <= column < getBoardSize()))
     */
    public String getLettersAt(int row, int column);

    /**
     * @returns <code>true</code> if <code>word</code> is contained
     * in this board. If <code>word</code> is <code>null</code>,
     * then this method must return <code>false</code>.
     */
    public boolean containsWord(String word);
    
    /**
     * @modifies shuffleListeners
     * @throws IllegalArgumentException if the listener is null
     * @returns <code>true</code> if this ShuffleListener was not
     * already listening for shuffle events
     */
    public boolean addShuffleListener(ShuffleListener l);

    /**
     * @modifies shuffleListeners
     * @throws IllegalArgumentException if the listener is null
     * @returns <code>true</code> if this ShuffleListener was
     * previously listening for shuffle events,
     * but is no longer listening for shuffle events
     */    
    public boolean removeShuffleListener(ShuffleListener l);

}
