package ps6.model;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * <code>Cube</code> represents a Cube in a Boggle board.
 * Immutable.
 *
 * A Cube is a sequence of Face objects of length NUM_FACES
 * @specfield Face [NUM_FACES] faces;
 * @author mbolin
 * @author cwo (modified for Java 1.5)
 */
public final class Cube {

    public static final int NUM_FACES = 6;
    
    private static final Set<Cube> CUBES = new HashSet<Cube>();
    
    public static final Cube CUBE_0 = new Cube(Face.createFace("E"),
            Face.createFace("E"), Face.createFace("F"), Face.createFace("H"),
            Face.createFace("I"), Face.createFace("Y"));

    public static final Cube CUBE_1 = new Cube(Face.createFace("D"),
            Face.createFace("E"), Face.createFace("N"), Face.createFace("O"),
            Face.createFace("S"), Face.createFace("W"));

    public static final Cube CUBE_2 = new Cube(Face.createFace("G"),
            Face.createFace("I"), Face.createFace("L"), Face.createFace("R"),
            Face.createFace("U"), Face.createFace("W"));

    public static final Cube CUBE_3 = new Cube(Face.createFace("E"),
            Face.createFace("G"), Face.createFace("I"), Face.createFace("K"),
            Face.createFace("L"), Face.createFace("Y"));

    public static final Cube CUBE_4 = new Cube(Face.createFace("A"),
            Face.createFace("D"), Face.createFace("E"), Face.createFace("N"),
            Face.createFace("V"), Face.createFace("Z"));

    public static final Cube CUBE_5 = new Cube(Face.createFace("A"),
            Face.createFace("B"), Face.createFace("I"), Face.createFace("L"),
            Face.createFace("T"), Face.createFace("Y"));

    public static final Cube CUBE_6 = new Cube(Face.createFace("E"),
            Face.createFace("G"), Face.createFace("I"), Face.createFace("N"),
            Face.createFace("T"), Face.createFace("V"));

    public static final Cube CUBE_7 = new Cube(Face.createFace("E"),
            Face.createFace("L"), Face.createFace("P"), Face.createFace("S"),
            Face.createFace("T"), Face.createFace("U"));

    public static final Cube CUBE_8 = new Cube(Face.createFace("A"),
            Face.createFace("B"), Face.createFace("J"), Face.createFace("M"),
            Face.createFace("O"), Face.createFace("Qu"));

    public static final Cube CUBE_9 = new Cube(Face.createFace("E"),
            Face.createFace("H"), Face.createFace("I"), Face.createFace("N"),
            Face.createFace("P"), Face.createFace("S"));

    public static final Cube CUBE_10 = new Cube(Face.createFace("A"),
            Face.createFace("C"), Face.createFace("E"), Face.createFace("L"),
            Face.createFace("R"), Face.createFace("S"));

    public static final Cube CUBE_11 = new Cube(Face.createFace("A"),
            Face.createFace("A"), Face.createFace("C"), Face.createFace("I"),
            Face.createFace("O"), Face.createFace("T"));

    public static final Cube CUBE_12 = new Cube(Face.createFace("A"),
            Face.createFace("H"), Face.createFace("M"), Face.createFace("O"),
            Face.createFace("R"), Face.createFace("S"));

    public static final Cube CUBE_13 = new Cube(Face.createFace("D"),
            Face.createFace("K"), Face.createFace("N"), Face.createFace("O"),
            Face.createFace("T"), Face.createFace("U"));

    public static final Cube CUBE_14 = new Cube(Face.createFace("B"),
            Face.createFace("F"), Face.createFace("I"), Face.createFace("O"),
            Face.createFace("R"), Face.createFace("X"));

    public static final Cube CUBE_15 = new Cube(Face.createFace("A"),
            Face.createFace("C"), Face.createFace("D"), Face.createFace("E"),
            Face.createFace("M"), Face.createFace("P"));    

    private final Face[] faces = new Face[NUM_FACES];
    
    /**
     * @requires none of the faces are null
     */
    private Cube(Face f0, Face f1, Face f2, Face f3, Face f4, Face f5) {
        faces[0] = f0;
        faces[1] = f1;
        faces[2] = f2;
        faces[3] = f3;
        faces[4] = f4;
        faces[5] = f5;
        CUBES.add(this);
    }

    public Face getFace(int index) {
        if (index < 0) {
            throw new IllegalArgumentException("index cannot be negative");
        } else if (index > NUM_FACES) {
            throw new IllegalArgumentException("index exceeds " + NUM_FACES);
        }
        return faces[index];
    }
    
    public static Iterator<Cube> allCubes() {
        return Collections.unmodifiableSet(CUBES).iterator();
    }
    
    public String toString() {
    	StringBuffer sb = new StringBuffer();
    	sb.append("Cube[");
    	for (int i = 0; i < faces.length; i++) {
    		sb.append(faces[i]).append(",");
    	}
    	sb.append("]");
    	return sb.toString();
    }

}
