
package ps6.model;

/**
 * <code>CubeBucket</code> is a bucket in the Boggle tray that holds a Cube.
 * 
 * The Cube that it holds may change over the existence of the CubeBucket,
 * so CubeBucket is mutable.
 *
 * A CubeBucket holds a cube with a particular face showing
 * 
 * @specfield Cube cube;
 * @specfield int face;
 * 
 * abstract invariant: 0 <= face < Cube.NUM_FACES
 * 
 * @author mbolin
 */
public class CubeBucket {

    private Cube cube;
    
    private int index;
    
    private int face;
    
    public CubeBucket(int index) {
        this.index = index;
    }
    
    CubeBucket(int index, Cube cube, int face) {
    	this.index = index;
    	this.cube = cube;
    	this.face = face;
    }
    
    public Cube getCube() {
        return cube;
    }
    
    public void setCube(Cube cube) {
        this.cube = cube;
    }
    
    public int getFace() {
        return face;
    }
    
    public void setFace(int face) {
        this.face = face;
    }
    
    public String getDisplayLetters() {
        return cube.getFace(face).getDisplayLetters();
    }
    
    public String getCapitalLetters() {
        return cube.getFace(face).getCapitalLetters();
    }
    
    public boolean equals(Object o) {
        if (!(o instanceof CubeBucket)) {
            return false;
        }
        CubeBucket cb = (CubeBucket)o;
        return this.index == cb.index;
    }
    
    public int hashCode() {
        return 17 * index;
    }
    
    public String toString() {
    	return "BUCKET " + index;
    }
    
}
